## Copyright (C) 2001 Red Hat, Inc.
## Copyright (C) 2001 Than Ngo <than@redhat.com>
## Copyright (C) 2001 Harald Hoyer <harald@redhat.com>
## Copyright (C) 2001 Philipp Knirsch <pknirsch@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import string
import commands
import math
import NC_functions

if not "/usr/lib/rhs/python" in sys.path:
    sys.path.append("/usr/lib/rhs/python")

import Conf
import ConfSMB
import NCHardwareList
from NC_functions import *
from NC_functions import _
import DeviceList
import NCDialup
import NCCipe

class ConfDevice(Conf.ConfShellVar):
    def __init__(self, name):
        new = false
        if not os.access(SYSCONFDEVICEDIR + 'ifcfg-' + name, os.R_OK):
            new = true
            
        Conf.ConfShellVar.__init__(self, SYSCONFDEVICEDIR + 'ifcfg-' + name)
        self.chmod(0600)

        if new:
            self.rewind()
            self.insertline("# Please read /usr/share/doc/initscripts-*/sysconfig.txt")
            self.nextline()
            self.insertline("# for the documentation of these parameters.");
            self.rewind()
            
class ConfRoute(Conf.ConfShellVar):
    def __init__(self, name):
        Conf.ConfShellVar.__init__(self, SYSCONFDEVICEDIR + name + '.route')
        self.chmod(0600)

class Device(DeviceList.Device_base):
    keydict = { 'Device' : 'DEVICE',
                'Name' : 'NAME',
                'OnBoot' : 'ONBOOT',
                'IP' : 'IPADDR',
                'Netmask' : 'NETMASK',
                'Gateway' : 'GATEWAY',
                'Hostname' : 'DEVHOSTNAME',
                'Domain' : 'DOMAIN',
                'BootProto' : 'BOOTPROTO',
                'Type' : 'TYPE',
                'HardwareAddress' : 'HWADDR',
                }

    boolkeydict = { 'OnBoot' : 'ONBOOT',
                    'AllowUser' : 'USERCTL',
                    'AutoDNS' : 'PEERDNS',
                    }
        
    def __init__(self, list = None, parent = None):
        DeviceList.Device_base.__init__(self, list, parent)        

    def apply(self, other):
        if not other:
            self.unlink()
            return
        # ApplyList
        self.setDeviceId(other.getDeviceId())
        self.setName(other.getName())
        self.setDevice(other.getDevice())
        self.setAlias(other.getAlias())
        self.setType(other.getType())
        self.setOnBoot(other.getOnBoot())
        self.setAllowUser(other.getAllowUser())
        self.setBootProto(other.getBootProto())
        self.setIP(other.getIP())
        self.setNetmask(other.getNetmask())
        self.setGateway(other.getGateway())
        self.setHostname(other.getHostname())
        self.setDomain(other.getDomain())
        self.setAutoDNS(other.getAutoDNS())
        self.setHardwareAddress(other.getHardwareAddress())
        if self.createStaticRoutes():
            self.StaticRoutes.apply(other.getStaticRoutes())
        if self.createCipe():
            self.Cipe.apply(other.getCipe())
        if self.createWireless():
            self.Wireless.apply(other.getWireless())
        if self.createDialup():
            self.Dialup.apply(other.getDialup())

    def createDialup(self):
        if self.Type:
            if self.Type == MODEM:
                if (self.Dialup == None) \
                   or not isinstance(self.Dialup, NCDialup.ModemDialup):
                    self.Dialup = NCDialup.ModemDialup(None, self)
            elif self.Type == ISDN:
                if (self.Dialup == None) \
                   or not isinstance(self.Dialup, NCDialup.IsdnDialup):
                    self.Dialup = NCDialup.IsdnDialup(None, self)
            elif self.Type == DSL:
                if (self.Dialup == None) \
                   or not isinstance(self.Dialup, NCDialup.DslDialup):
                    self.Dialup = NCDialup.DslDialup(None, self)
            else:
                self.Dialup = None
                
            return self.Dialup

        else:
            raise TypeError, _("Device type not specified")

    def createCipe(self):
        if self.Type:
            if self.Type == CIPE:
                DeviceList.Device_base.createCipe(self)
            else: self.Cipe = None
                
            return self.Cipe

        else:
            raise TypeError, _("Device type not specified")

    def createWireless(self):
        if self.Type:
            if self.Type == WIRELESS:
                DeviceList.Device_base.createWireless(self)
            else: self.Wireless = None
                
            return self.Wireless

        else:
            raise TypeError, _("Device type not specified")


    def load(self, name):
        from netconfpkg.NCDeviceList import getDeviceList
        conf = ConfDevice(name)
        rconf = ConfRoute(name)

        if not conf.has_key("DEVICE"):
            aliaspos = string.find(name, ':')
            if aliaspos != -1:
                # ok, we have to inherit all other data from our master
                for dev in getDeviceList():
                    if dev.Device == name[:aliaspos]:
                        self.apply(dev)
                        break

            self.Device = name
         
        self.DeviceId = name

        for selfkey in self.keydict.keys():
            confkey = self.keydict[selfkey]
            if conf.has_key(confkey):
                self.__dict__[selfkey] = conf[confkey]

        for selfkey in self.boolkeydict.keys():
            confkey = self.boolkeydict[selfkey]
            if conf.has_key(confkey):
                if conf[confkey] == 'yes':
                    self.__dict__[selfkey] = true
                else:
                    self.__dict__[selfkey] = false            
            elif not self.__dict__.has_key(selfkey):
                self.__dict__[selfkey] = false                            
            
        if not self.Gateway:
            try:
                cfg = Conf.ConfShellVar(SYSCONFNETWORK)
                if cfg.has_key('GATEWAY'):
                    gw = cfg['GATEWAY']
                    
                    if gw and self.Netmask:                    
                        network = commands.getoutput('ipcalc --network ' + str(self.IP) \
                                            + ' ' + str(self.Netmask) + \
                                            ' 2>/dev/null')
                        
                        out = commands.getoutput('ipcalc --network ' + str(gw) + ' ' \
                                        + str(self.Netmask) + ' 2>/dev/null')
                        
                        if out == network:
                            self.Gateway = str(gw)
                            
            except (OSError, IOError), msg:
                pass

        try:
            aliaspos = string.find(self.Device, ':')
            if aliaspos != -1:
                self.Alias = int(self.Device[aliaspos+1:])
                self.Device = self.Device[:aliaspos]
        except TypeError:
            raise TypeError, _("Device not specified or alias not a number!")

        if not self.Type or self.Type == "" or self.Type == "Unknown":
            hwlist = NCHardwareList.getHardwareList()
            for hw in hwlist:
                if hw.Name == self.Device:
                    self.Type = hw.Type
                    break
                    
        if (not self.Type or self.Type == "" or self.Type == "Unknown" ) \
           and self.Device:            
            self.Type = NC_functions.getDeviceType(self.Device)

        #print "Creating Dialup"
        dialup = self.createDialup()
        if dialup:
            #print "Loading Dialup"
            dialup.load(conf)

        cipe = self.createCipe()
        if cipe:
            cipe.load(conf)

        wireless = self.createWireless()
        if wireless:
            wireless.load(conf)

        num = len(rconf.keys())
        self.createStaticRoutes()

        if self.Type == CTC or self.Type == IUCV:
            if conf['MTU']:
                self.Mtu = conf['MTU']
                
        if math.fmod(num, 3) != 0:
            print (_("Static routes file for %s has not vaild format")) % name
        else:
            for p in xrange(0, num/3):
                i = self.StaticRoutes.addRoute()
                route = self.StaticRoutes[i]
                route.Address = rconf['ADDRESS'+str(p)]
                route.Netmask = rconf['NETMASK'+str(p)]
                route.Gateway = rconf['GATEWAY'+str(p)]
        self.commit(changed=false)
                
    def save(self):
        self.commit()

        conf = ConfDevice(self.DeviceId)
        conf.fsf()
        
        if not self.Cipe and self.BootProto == None:
            self.BootProto = 'dhcp'

        if self.BootProto:
            self.BootProto = string.lower(self.BootProto)

        for selfkey in self.keydict.keys():
            confkey = self.keydict[selfkey]
            if self.__dict__[selfkey]:
                conf[confkey] = str(self.__dict__[selfkey])
            else: conf[confkey] = ""

        if self.Alias != None:
            conf['DEVICE'] = str(self.Device) + ':' + str(self.Alias)

        for selfkey in self.boolkeydict.keys():
            confkey = self.boolkeydict[selfkey]
            if self.__dict__[selfkey]:
                conf[confkey] = 'yes'
            else:
                conf[confkey] = 'no'

        # Recalculate BROADCAST and NETWORK values if IP and netmask are
        # present (#51462)
        if self.IP and self.Netmask:
            broadcast = commands.getoutput('ipcalc --broadcast ' + str(self.IP) \
                                            + ' ' + str(self.Netmask) + \
                                            ' 2>/dev/null')
            if broadcast:
                conf['BROADCAST'] = broadcast[10:]

            network = commands.getoutput('ipcalc --network ' + str(self.IP) \
                                            + ' ' + str(self.Netmask) + \
                                            ' 2>/dev/null')
            if network:
                conf['NETWORK'] = network[8:]

        if self.Type == CTC or self.Type == IUCV:
            if not self.Mtu: self.Mtu = 1492
            conf['MTU'] = self.Mtu
            if conf['GATEWAY']:
                conf['REMIP'] = conf['GATEWAY']

        if self.Dialup:
            self.Dialup.save(conf)

        if self.Cipe:
            self.Cipe.save(conf)

        if self.Wireless:
            self.Wireless.save(conf)

        if self.StaticRoutes and len(self.StaticRoutes) > 0:
            rconf = ConfRoute(self.DeviceId)
            p = 0
            for route in self.StaticRoutes:
                rconf['ADDRESS'+str(p)] = route.Address
                rconf['NETMASK'+str(p)] = route.Netmask
                rconf['GATEWAY'+str(p)] = route.Gateway
                p = p + 1
            rconf.write()

        # Do not clear the non-filled in values for Wireless Devices
        # Bugzilla #52252
        if not self.Wireless:
            for i in conf.keys():
                if not conf[i] or conf[i] == "": del conf[i]

        conf.write()
